# Datasheet Format

```{note}
Prior to June 2024 CACE used the datasheet version 4.0 text format. This format is now deprecated, please export your datasheet in the new YAML format.
```

The CACE datasheet contains the specification of your design and other important information. It acts as both documentation for the specifiaction but also as input for CACE. 

The following sections describe the structure of the datasheet.

## Metadata

Example entry:

```yaml
name: ota-5t
description: A simple 5 transistor OTA
commit: n/a
PDK: sky130A
cace_format: 5.0
```

- `name:	<string>`
    > Name of the design (cell name)

- `description:	<string>`
    > Description of the design

- `commit:  <hash-value>`
    > Commit hash of the repository of this IP

- `PDK:	<string>`
    > Name of the PDK (no spaces)

- `cace_format:	<string>`
    > Format version of the datasheet. Current version: `5.0`.

## Authorship

Example entry:

```yaml
authorship:
  designer: Leo Moser
  company: Efabless
  creation_date: May 27, 2024
  license: Apache 2.0
```

- `designer: <name>`
    > The person to whom the design is attributed.

- `company:	<name>`
    > The name of a company, if applicable.  This is generally for
    reference and may be replaced by "institution" or other key.

- `creation_date:	<datestamp>`
    > Date that the CACE setup was made, or any meaningful date
    for the creation of the design.

- `modification_date: <datestamp>`
    > Generally handled by the tools to specify when the characterization
    file was last updated.

- `license:	<string>`
    > A known license type, such as "Apache 2.0".

## Paths

The paths section is used to tell CACE where to find certain files of your design or where to generate files for simulation.

Example entry:

```yaml
paths:
  root: ..
  documentation: doc
  schematic: xschem
  layout: gds
  netlist: netlist
  testbench: cace
  simulation: ngspice
  plots: plots
  logs: ngspice/log
  reports: reports
```

- `documentation:	<path>`
    > Location of any documentation for the design.

- `schematic:	<path>`
	> Location of any schematics (xschem) and symbols for the design.

- `verilog:	<path>`
	> Location of any structural verilog used in the design.

- `magic:		<path>`
	> Location of any layout in magic (.mag) format.

- `layout:		<path>`
	> Location of any layout in GDS (.gds or .gds.gz) format.

- `netlist:	<path>`
	> Location of all SPICE netlists.
	This netlist is usually automatically generated by cace_regenerate.py,
	with subdirectories for each netlist type according to the netlist
	source (see "-source" option):  "schematic", "layout" (for LVS),
	"pex" (C-parasitic extracted layout), and "rcx" (R-C-parasitic
	extracted layout).

- `testbench:	<path>`
	> Location of schematics and/or netlists used by the CACE system for
	running characterization simulations.

- `simulation:	<path>`
	> Location of files used for simulation (e.g., netlists created by
	CACE through variable substitution into the testbench templates)
	and simulation results.

- `reports:	<path>`
	> Reports and log files generated by CACE.

- `root:		<path>`
	> This is the location of the project and the root of all the other
	paths in this dictionary.  It is automatically inserted by CACE.

## Dependencies

Each entry is a project on which the design under test is dependent.

Example entry:

```yaml
dependencies:
  sky130_ef_ip__samplehold:
    path: ..
    repository: https://github.com/RTimothyEdwards/sky130_ef_ip__samplehold
    commit: N/A
  ...
```

- `key:	<string>`
	> Name of the dependency

- `path: <path>`
	> Path to the dependency (may be relative or absolute, but
	preferably relative for portability).

- `repository: <url>`
	> Location of the project repository of the dependency, so that
	if the dependency does not exist, it can be cloned into the
	target path on demand.
	
- `commit: <hash>`
	> Commit hash of the repository of the dependency to use in this
	project.  TBD:  Various values that define actions such as updating
	the repository.

## Pins

The pins of the design, mainly used for documentation purposes. The names of the pins are checked against the schematic.

Example entry:

```yaml
pins:
  VDD:
    description: Power supply
    direction: inout
    type: power
    Vmin: 1.7
    Vmax: 1.9
  ...
```

- `name:	<string>`
	> The pin name.  Must match the name on the schematic or netlist.
	Vectors should be indexed with ":" and will be expanded when
	needed.  "b7:0" expands to "b7, b6, b5, ..." while "b[7:0]"
	expands to "b[7], b[6], b[5], ..." (Note that this is more
	flexible than the convention for condition names described
	below.)

- `description:	<string>`
	> Text description of the pin. May contain spaces.

- `type:	<string>`
	> The type may be one of "digital", "signal", "power", or "ground".
	
- `direction:	<string>`
	> The direction may be one of "input", "output", or "inout".

- `Vmin:	<number>|<expression>`
	> The pin minimum voltage may be a value or may be referenced to
	another pin;  and may be referenced to another pin with an offset
	(e.g., "vss - 0.3").

- `Vmax:	<number>|<expression>`
	> The pin minimum voltage may be a value or may be referenced to
	another pin;  and may be referenced to another pin with an offset
	(e.g., "vdd + 0.3").

## Default Conditions

The default conditions under which to evaluate the parameters.

Example entry:

```yaml
default_conditions:
  corner:
    display: Corner
    description: The corner of the wafer
    typical: tt
    unit: V
  ...
```

- `name: <string>`
	> The name of the condition;  this name is meaningful because it
	will match a variable name used in a schematic or netlist.  The
	representation in the netlist is always ${<string>} to prevent
	accidental substitutions of matching strings.  Any set of signals
	can be bundled, but the delimiters for the bundle must be brackets;
	e.g., b[7:0], with single bits called out as, e.g., b[0].

- `description: <string>`
	> A description of the condition.

- `display: <string>`
	> A short, typically one-word value to display for the condition
	in a datasheet or GUI.

- `unit: <string>`
	> The unit of measure used to display the condition value.

- `typical: <value>`
	> If present, the default typical value of the condition.

- `maximum: <value>`
	> If present, the default maximum value of the condition.

- `minimum: <value>`
	> If present, the default minimum value of the condition.

- `enumerate: <values>`
	> If present, instead of min/typ/max values, the values are
	enumerated from a space-separated list supplied in <values>
	(see above; long lists may be backslash-newline terminated).

- `step: linear|logarithmic`
	> If not present, then only values min/typ/max are evaluated.
	If present, then values are automatically enumerated from
	min to max inclusive either in linear (default) or logarithmic
	progression.  If typ exists and is not in the enumeration,
	then it is evaluated in addition.

- `stepsize: <value>`
	> If not present, then a stepsize of 1 is assumed for linear
	enumeration or 2 for logarithmic enumeration.  Otherwise,
	the enumeration steps by <value> additive values for linear
	enumeration or <value> multiplicative values for logarithmic
	enumeration.

## Electrical Parameters

Example entry:

```yaml
electrical_parameters:
  A0:
    description: DC gain
    display: A0
    unit: V/V
    spec:
      minimum:
        value: '3'
        fail: true
      typical:
        value: any
      maximum:
        value: any
    simulate:
      ngspice:
        template: ota-5t_tb.spice
        format: ascii
        suffix: .data
        variables: [result, 'null', 'null']
    conditions:
      vdd:
        typical: '1.8'
      vcm:
        typical: '0.9'
      corner:
        enumerate: [tt, ff, ss]
```

- `name: <string>`
	> A name for the electrical parameter (used only for bookkeeping
	purposes).

- `description: <string>`
	> A description of the electrical parameter (may contain spaces)

- `display: <string>`
	> A short description to display for the electrical parameter
	on a datasheet or GUI.
	
- `unit: <string>`
	> The unit measurement of the electrical parameter result value.
	Same format as "unit" for conditions.

- `spec: <dictionary>`
	> The target specification for the electrical parameter (see below).

- `results: <list>`
	> The results corresponding to the spec, with dictionary entries
	similar to "spec".  It is a list because specs may be measured
	at the schematic stage of design, layout for LVS, or layout
	with parasitics (see "results" dictionary definition, below).
	
- `simulate: <list>`
	> How to collect data for measurement.  Normally this list is a
	single item, as there is only one way to simulate a parameter.
	However, cosimulations or other complex setups may require
	additional entries.  Each entry in the list is a tool or script
	to be run to produce simulation results.

- `measure: <list>`
	> How to measure the electrical parameter or collect data for
	measurement.  Each entry in the list is a tool or script to
	be run in sequence (see "measure" dictionary definiton, below).

- `conditions: <dictionary>`
	> All the conditions to be applied to the measurement of the
	electrical parameter that are considered to be fixed with respect
	to the electrical parameter result (see "variables" below).

- `variables: <list>`
	> Enumerates the variables that are enumerated within the simulation
	and output but which are not conditions (e.g., time).  The 'result'
	variable (the one representing the electrical parameter) is already
	specified in the parameter and does not need to be in the 'variables'
	list.

- `plot: <dictionary>`
	> Describes how to make a data plot for the electrical parameter
	(where result is plotted rather than generating a single set
	of min/typ/max values).  Requires a corresponding "variables"
	list.

### Conditions

Example entry:

```yaml
    conditions:
      vdd:
        typical: '1.8'
      vcm:
        typical: '0.9'
      corner:
        enumerate: [tt, ff, ss]
```

The "conditions" dictionary entries are the same as the
"default_conditions" dictionary, above.  The "default_conditions"
dictionary is applied to every condition before adding or
overwriting with the specific conditions listed for the
electrical parameter.  Conditions are only meaningful when the
condition name exists in the netlist to be simulated or the file
to be evaluated (see "simulate" and "measure" entries).

### Specification

Example entry:

```yaml
    spec:
      minimum:
        value: '3'
        fail: true
      typical:
        value: any
      maximum:
        value: any
```

- ```yaml
  minimum:
	value: <value>|any
	fail: <bool>
	measurement: [<calculation>-<limit>]
  ```
	> Specifies the target value for the electrical parameter minimum
	value.  If not specified, then no minimum is measured.  If "any"
	is given as the value, then the minimum is measured but there
	is no target value.
	By default, the electrical parameter is marked as failing if the
	measured value is out of spec. By setting `fail: false`, it
	is not checked agains the limit. If <calculation>-<limit>
	is specified, then it overrides	the default calculation of "minimum-above"

- ```yaml
  typical:
	value: <value>|any
	fail: <bool>
	measurement: [<calculation>-<limit>]
  ```
	> Specifies the target value for the electrical parameter typical
	value.  If not specified, then no typical is measured.  If "any"
	is given as the value, then the typical is measured but there
	is no target value.  If "fail" is also specified as "true", then electrical
	parameter is marked as failing if the measured value is out of
	spec (failing on a typical value is rarely if ever used).
	If <calculation>-<limit> is specified, then it overrides the
	default calculation of "average-exact"

- ```yaml
  maximum:
	value: <value>|any
	fail: <bool>
	measurement: [<calculation>-<limit>]
  ```
	> Specifies the target value for the electrical parameter maximum
	value.  If not specified, then no maximum is measured.  If "any"
	is given as the value, then the maximum is measured but there
	is no target value.  If "fail" is also specified as "true", then electrical
	parameter is marked as failing if the measured value is out of
	spec.  If <calculation>-<limit> is specified, then it overrides
	the default calculation of "maximum-below"

### Simulate

Currently, only ngspice is supported as simulator, therefore the dictionary key has to be ngspice.

Example entry:

```yaml
    simulate:
      ngspice:
        template: ota-5t_tb.spice
        format: ascii
        suffix: .data
        variables: [result, 'null', 'null']
```

- `template: <string>`
	> This specifies the name of the file that is used as the template for
	simulation.  The conditions for the specific measurement are
	determined, and the condition values are variable-substituted in the
	template, and then the resulting set of files is passed to the tool.

- `format: <string>`
	> This specifies the format of the simulation output. Currently,
	only "ascii" is supported (however, "raw" for SPICE raw files will be
	supported in the future, and "vcd" files that may be produced from
	verilog cosimulation).
	
- `suffix: <string>`
    > suffix is the filename suffix of the output file, which must have 
    the same root name as the file passed to the tool. (For example if 
    template is "ota-5t\_tb.spice" and suffix ".data", then CACE expects 
    "ota-5t\_tb.data" to be created during simulation.)

	The format line informs CACE how to merge results from multiple
	simulation runs for the same electrical parameter by describing
	the format of the tool's output file and its contents.
	
- `variables: <list>`
	> variables is a list consisting of one of "result", "time" or "null".
	"result" is the value that will be used for comparison against the spec.
	"time" is a time value that may be used for plotting. "null" for any value
	that is not evaluated, or any condition or variable name. If any
	condition name appears in the format string, then there must be
	enough lines of output in the file to match the number of conditions
	simulated.  This is used for sweeps and plots.

- `collate: <condition>`
	> This specifies that the simulation output should be collated from
	runs over all the specified conditions before passing the result
	to the first "measure" handler.

### Variables

Example entry:

```yaml
    variables:
      time:
        display: Time
        unit: ms
      Vinp:
        display: Vinp
        unit: V
      result:
        display: Vout
        unit: V
```

A variable is any property that affects the electrical parameter
result but which is not a condition.  A variable may be used in
a plot to graph the result vs. something that is not a condition.
Otherwise, all variables must be eliminated during the course of
evaluation.  Note:  Use of "collate" in a "simulate" dictionary
will force any condition in the collated list to become a
variable, as if it were produced by the simulation instead of
being passed to it.

- `name: <string>`
	> The name is a placeholder for bookkeeping, except for string
	"result" which indicates that the condition in this dictionary
	is the result vector to be plotted.

- `display: <string>`
	> This value is text to display on the graph, which may appear
	in a key if the value is not for data corresponding to the
	graph X or Y axis.

- `unit: <value>`
	> This is the unit that will be displayed after the display
	string.


### Plots

Example entry:

```yaml
    plot:
      filename: transient.png
      xaxis: time
      yaxis: result
```

- `filename: <string>`
	Name of a graphic format and filename to use for graph output.
	Plots are done with python matplotlib, so it must be a format
	known to matplotlib.

- `type: <string>`
	The type of plot to make.  If this record is missing from the
	dictionary, then plot type "xyplot" is assumed by default.
	Otherwise, the value should be one of "xyplot", "histogram",
	"semilogx", "semilogy", or "loglog".

- `xaxis: <name>`
	Condition to be plotted on the graph X axis, as determined
	from the condition related to <name> in the "variables"
	section.  The X-axis label is taken from the variable entry
	as well.

- `yaxis: <name>`
	Condition to be plotted on the graph Y axis, as determined
	from the condition related to <name> in the "variables"
	section.  The Y-axis label is taken from the variable entry
	as well.

- `title: <string>`
	A title for the graph.

Plots are made from measured columnar data which may be from a "wrdata"
command in ngspice, an "echo" statement directed to a file, or a raw file;
regardless, each column will be either a condition or a result.  Conditions
produced by plot-generating tools may contain vectors not in the listed
conditions for the electrical parameter, such as TIME for time or TRACE for
the result.

### Measure

"measure" dictionary:

```yaml
    measure:
    - tool: octave
      filename: dnl.m
```

Additional entries are tools that operate on the output of
the simulator to produce the final results for the datasheet.
They may be number-crunching tools like "octave" for compute-
intensive evaluations, or they may be a script like python
(using numpy or scipy) or bash.  The purpose of "measure" is
to modify the result produced by the simulation, including
removing variables from the result list.

- `calc: <string>`
	> This specifies an internal calculation understood by CACE to apply
	to the testbench result (see "caculations", below).

- `tool: <string>`
	> This specifies an external tool to use to make the measurement,
	typically octave.  It must also specify a filename of a script
	to run (see below).  

- `filename: <string>`
	> The filename is the name of a script to pass to the tool given as the
	value to the "tool" keyword (e.g., a ".m" script to pass to octave).
	The script must accept one argument, which is a filename consisting
	of the parameter name followed by ".dat".  It is a list of all
	results and the set of conditions for each.  Currently, ".dat" is in
	octave format, but should support JSON format for python scripts.
	The output of the "evaluate" tool is a single list of results that
	replaces (in order) the result list passed to the tool.  Currently,
	the output is passed in stdout, and is a list of "result" values
	that replaces the existing "result" values in the testbench.

Internal calculation (measurement) methods:

	> time:
	remove:
	rebase:
	abs:
	negate:
	add:
	subtract:
	multiply:
	clip:
	mean:
	risingedge:
	fallingedge:
	stabletime:
	inside:

## Physical Parameters

Example entry:

```yaml
physical_parameters:
  area:
    description: Total circuit layout area
    display: Area
    unit: µm²
    spec:
      maximum:
        value: '100'
        fail: true
    evaluate: cace_area
```

The physical parameters list has the same keywords as the
electrical parameters, except that the parameters are
physical measurements of the layout such as area, width,
height, max metal layer, and may include number of DRC
or LVS errors.

Dictionary keys that are shared with "electrical_parameters"
are: name, status, description, display, unit, and spec.

The "evaluate" dictionary is unique to physical parameters
and takes the place of "simulate" and "measure" for electrical
parameters (see below).

Internal physical parameter (evaluation) tools:

- `cace_drc`:
    > Returns the number of DRC errors found in the design.
	The result is generally useful only as being zero or
	non-zero.

- `cace_lvs`:
    > Returns the number of LVS errors found in the design.
	The result is generally useful only as being zero or
	non-zero.

- `cace_area`:
    > Returns the area of the layout.

- `cace_width`:
    > Returns the width of the layout.

- `cace_length`:
    >Returns the length of the layout.

### Evaluate

Example entry:

```yaml
    evaluate: cace_area
```

The "evaluate" dictionary is unique to the physical_parameters
dictionaries.  It describes how to obtain a specific physical
or other non-electrical parameter, such as cell size or number
of DRC or LVS errors.  Running the tool must produce a single
value as output, which is then checked against the spec to
provide a pass/fail result.

- `tool: <string>`
	> The name of a tool to use for the evaluation.  This may be an
	internal procedure known to CACE (see list below).  If not, then
	it must be a standard application or interpreter (such as python).
	The filename (see below) is passed to the tool as the argument.
	Any options to pass to the tool should be included in the string.

- `filename: <name>`
	> The name of a script or filename to pass to the tool specified
	by "tool", if the tool is not an internal procedure.  The file
	<name> should exist in the testbench directory.
